stop_words = {
    "着呢",
    "doing",
    "大批",
    "与",
    "serious",
    "likely",
    "every",
    "“",
    "right",
    "嘎登",
    "如果",
    "am",
    "非但",
    "specified",
    "就是",
    "进入",
    "might",
    "尔后",
    "appear",
    "先後",
    "关于",
    "definitely",
    "根据",
    "me",
    "my",
    "et",
    "clearly",
    "她的",
    "needs",
    "每天",
    "soon",
    "哟",
    "不怕",
    "little",
    "中小",
    "如何",
    "必要",
    "最近",
    "照着",
    "充分",
    "useful",
    "seem",
    "呼哧",
    "respectively",
    "myself",
    "属于",
    "宣布",
    "gives",
    "广大",
    "is",
    "thence",
    "你的",
    "therefore",
    "exactly",
    "contain",
    "edu",
    "果然",
    "扩大",
    "嗡嗡",
    "个人",
    "咳",
    "against",
    "反过来",
    "most",
    "哪",
    "一样",
    "almost",
    "後面",
    "规定",
    "不比",
    "cant",
    "不仅",
    "none",
    "besides",
    "由于",
    "known",
    "当前",
    "toward",
    "随着",
    "inner",
    "乎",
    "迅速",
    "你们",
    "您",
    "won't",
    "这会儿",
    "换句话说",
    "outside",
    "went",
    "防止",
    "进步",
    "towards",
    "except",
    "现代",
    "each",
    "a's",
    "are",
    "反过来说",
    "may",
    "不过",
    "让",
    "前面",
    "hello",
    "theirs",
    "表示",
    "有时",
    "首先",
    "大大",
    "some",
    "help",
    "下去",
    "不敢",
    "各",
    "特点",
    "先生",
    "hereupon",
    "甚么",
    "there's",
    "大量",
    "their",
    "value",
    "for",
    "not",
    "ours",
    "宁可",
    "几乎",
    "us",
    "latter",
    "每当",
    "罢了",
    "曾经",
    "thats",
    "省得",
    "这些",
    "别的",
    "换言之",
    "要求",
    "良好",
    "嘛",
    "如",
    "meanwhile",
    "这点",
    "纵使",
    "particularly",
    "大家",
    "actually",
    "集中",
    "different",
    "继续",
    "several",
    "do",
    "它的",
    "because",
    "还是",
    "自",
    "及",
    "next",
    "else",
    "whole",
    "嘘",
    "显然",
    "believe",
    "吧",
    "失去",
    "产生",
    "归",
    "加入",
    "as",
    "各级",
    "必须",
    "似的",
    "另一方面",
    "给",
    "慢说",
    "i'm",
    "okay",
    "anyhow",
    "丰富",
    "岂但",
    "nobody",
    "随",
    "immediate",
    "果真",
    "哪里",
    "与此同时",
    "直到",
    "sorry",
    "了解",
    "himself",
    "sup",
    "with",
    "各人",
    "以后",
    "indicate",
    "wasn't",
    "than",
    "以至于",
    "consider",
    "当然",
    "复杂",
    "hardly",
    "一",
    "多数",
    "即",
    "c's",
    "而言",
    "asking",
    "应当",
    "thoroughly",
    "via",
    "apart",
    "依",
    "gotten",
    "那麽",
    "happens",
    "起见",
    "假若",
    "哪怕",
    "now",
    "漫说",
    "比如",
    "论",
    "一天",
    "somehow",
    "哪个",
    "inasmuch",
    "假使",
    "amongst",
    "it's",
    "forth",
    "只限",
    "被",
    "somewhere",
    "旁人",
    "设若",
    "ought",
    "结果",
    "有着",
    "before",
    "完全",
    "可见",
    "it'd",
    "愿意",
    "非常",
    "最高",
    "tell",
    "深入",
    "我的",
    "并",
    "hopefully",
    "她",
    "overall",
    "有些",
    "qv",
    "彻底",
    "以",
    "should",
    "那会儿",
    "从",
    "现在",
    "哪儿",
    "过",
    "later",
    "把",
    "where",
    "宁肯",
    "we've",
    "一直",
    "那边",
    "方面",
    "i'd",
    "嘻",
    "且",
    "behind",
    "having",
    "it",
    "by",
    "yourselves",
    "aren't",
    "有利",
    "did",
    "下来",
    "最後",
    "哎呀",
    "看看",
    "wouldn't",
    "彼",
    "third",
    "及其",
    "i've",
    "纵令",
    "自己",
    "呜",
    "替",
    "hereby",
    "焉",
    "显著",
    "较",
    "反之",
    "relatively",
    "附近",
    "few",
    "哎哟",
    "try",
    "thereafter",
    "这就是说",
    "best",
    "consequently",
    "contains",
    "好象",
    "they",
    "什麽",
    "凭借",
    "thus",
    "由",
    "伟大",
    "待",
    "kept",
    "saw",
    "seeming",
    "不论",
    "otherwise",
    "则",
    "却不",
    "那里",
    "战斗",
    "say",
    "much",
    "适当",
    "后面",
    "especially",
    "怎么样",
    "继而",
    "引起",
    "行动",
    "要",
    "嗳",
    "巨大",
    "他",
    "下列",
    "临",
    "任",
    "某些",
    "可",
    "冲",
    "那时",
    "obviously",
    "因此",
    "叫",
    "左右",
    "其次",
    "hence",
    "与否",
    "专门",
    "need",
    "巩固",
    "总而言之",
    "wish",
    "old",
    "前者",
    "做到",
    "往",
    "一些",
    "总是",
    "course",
    "以来",
    "something",
    "使用",
    "最大",
    "these",
    "主要",
    "among",
    "somebody",
    "where's",
    "本着",
    "at",
    "么",
    "new",
    "along",
    "不然",
    "俺们",
    "即使",
    "望",
    "thank",
    "明确",
    "ourselves",
    "否则",
    "共同",
    "her",
    "him",
    "等",
    "注意",
    "let",
    "way",
    "的",
    "倘使",
    "aside",
    "向着",
    "无论",
    "嘎",
    "要么",
    "目前",
    "有点",
    "怎样",
    "out",
    "逐渐",
    "throughout",
    "个别",
    "之类",
    "enough",
    "我",
    "综上所述",
    "yet",
    "倘若",
    "if",
    "随著",
    "both",
    "不久",
    "never",
    "另",
    "为什么",
    "hadn't",
    "wherein",
    "don't",
    "becomes",
    "此时",
    "比方",
    "靠",
    "既然",
    "anyway",
    "决定",
    "一次",
    "well",
    "各位",
    "既是",
    "anyone",
    "fifth",
    "吱",
    "相对",
    "促进",
    "whether",
    "主张",
    "确定",
    "所",
    "後来",
    "吧哒",
    "go",
    "什么",
    "sub",
    "怎麽",
    "起",
    "哪些",
    "例如",
    "presumably",
    "重新",
    "thanks",
    "喏",
    "限制",
    "安全",
    "咋",
    "here's",
    "强调",
    "indicates",
    "她们",
    "就是说",
    "through",
    "nevertheless",
    "only",
    "哇",
    "倘",
    "不只",
    "不同",
    "并没有",
    "要不",
    "呗",
    "you're",
    "尤其",
    "ones",
    "herself",
    "afterwards",
    "you",
    "各自",
    "有效",
    "moreover",
    "example",
    "得出",
    "之所以",
    "另外",
    "然而",
    "转变",
    "不变",
    "如上所述",
    "若",
    "shall",
    "哩",
    "如若",
    "沿着",
    "non",
    "顺",
    "其他",
    "nd",
    "及时",
    "allows",
    "不光",
    "有力",
    "everything",
    "goes",
    "直接",
    "哦",
    "倘或",
    "人家",
    "完成",
    "quite",
    "吗",
    "强烈",
    "hi",
    "does",
    "当着",
    "明显",
    "容易",
    "虽",
    "unto",
    "various",
    "地",
    "four",
    "no",
    "why",
    "总的来说",
    "经常",
    "trying",
    "whereafter",
    "况且",
    "take",
    "re",
    "从事",
    "能够",
    "该",
    "ltd",
    "been",
    "given",
    "be",
    "转动",
    "anything",
    "只是",
    "we",
    "together",
    "以及",
    "转贴",
    "那么些",
    "时候",
    "像",
    "哎",
    "心里",
    "为主",
    "不一",
    "或者",
    "above",
    "possible",
    "wonder",
    "further",
    "ok",
    "knows",
    "we'd",
    "既",
    "何处",
    "呀",
    "逐步",
    "still",
    "多少",
    "运用",
    "ever",
    "因为",
    "least",
    "onto",
    "didn't",
    "为何",
    "大力",
    "吓",
    "since",
    "因",
    "谁",
    "i'll",
    "今天",
    "follows",
    "尚且",
    "毋宁",
    "其余",
    "zz",
    "紧接着",
    "everybody",
    "再说",
    "作为",
    "自各儿",
    "许多",
    "这边",
    "那么",
    "一起",
    "不如",
    "正如",
    "everyone",
    "普遍",
    "come",
    "哈哈",
    "看来",
    "came",
    "containing",
    "本",
    "就",
    "regards",
    "不要",
    "不拘",
    "咦",
    "willing",
    "考虑",
    "不够",
    "互相",
    "咱们",
    "associated",
    "alone",
    "three",
    "according",
    "恰恰相反",
    "entirely",
    "喔唷",
    "咚",
    "打",
    "sure",
    "除非",
    "who",
    "那",
    "already",
    "following",
    "无宁",
    "look",
    "当",
    "应该",
    "非徒",
    "此外",
    "用",
    "perhaps",
    "每",
    "相似",
    "加之",
    "起来",
    "一时",
    "特别是",
    "changes",
    "你",
    "so",
    "about",
    "an",
    "indicated",
    "哉",
    "那个",
    "总的来看",
    "between",
    "upon",
    "已经",
    "another",
    "provides",
    "first",
    "they'd",
    "以上",
    "ain't",
    "这么",
    "几时",
    "self",
    "几",
    "either",
    "或",
    "经",
    "上来",
    "具体说来",
    "之前",
    "below",
    "greetings",
    "then",
    "without",
    "可以",
    "一致",
    "出去",
    "although",
    "our",
    "而已",
    "gets",
    "俺",
    "here",
    "around",
    "若是",
    "所有",
    "受到",
    "为了",
    "总结",
    "任凭",
    "这么点儿",
    "而是",
    "至于",
    "除了",
    "可能",
    "以为",
    "其",
    "多",
    "广泛",
    "downwards",
    "seven",
    "了",
    "嗯",
    "看出",
    "按",
    "able",
    "we're",
    "see",
    "until",
    "各个",
    "其它",
    "you'd",
    "即便",
    "矣",
    "双方",
    "rather",
    "novel",
    "看到",
    "beyond",
    "不可",
    "anybody",
    "want",
    "have",
    "got",
    "每个",
    "然後",
    "仍然",
    "无法",
    "”",
    "into",
    "趁",
    "如此",
    "such",
    "on",
    "实际",
    "认识",
    "有著",
    "whither",
    "呜呼",
    "seriously",
    "己",
    "别",
    "when",
    "them",
    "甚至",
    "brief",
    "应用",
    "接著",
    "突然",
    "looks",
    "same",
    "及至",
    "严格",
    "一般",
    "莫若",
    "whoever",
    "不是",
    "》",
    "即或",
    "cannot",
    "he's",
    "尽管",
    "真是",
    "之后",
    "also",
    "相对而言",
    "至",
    "down",
    "noone",
    "绝对",
    "其一",
    "was",
    "同",
    "得到",
    "please",
    "last",
    "comes",
    "however",
    "真正",
    "hither",
    "always",
    "有的",
    "the",
    "而况",
    "usually",
    "尽",
    "whereas",
    "nor",
    "先后",
    "的话",
    "just",
    "become",
    "th",
    "etc",
    "除此之外",
    "specifying",
    "of",
    "知道",
    "we'll",
    "shouldn't",
    "哪天",
    "等等",
    "立即",
    "you'll",
    "上去",
    "he",
    "know",
    "欢迎",
    "维持",
    "兮",
    "冒",
    "如其",
    "高兴",
    "from",
    "tried",
    "neither",
    "corresponding",
    "表明",
    "相信",
    "这麽",
    "密切",
    "不问",
    "看见",
    "进而",
    "一片",
    "上下",
    "达到",
    "上述",
    "开展",
    "can't",
    "正常",
    "为着",
    "腾",
    "于是乎",
    "sensible",
    "哼",
    "开始",
    "组成",
    "阿",
    "lately",
    "纵然",
    "或是",
    "appropriate",
    "whose",
    "今年",
    "呸",
    "six",
    "呵",
    "有",
    "far",
    "eight",
    "这时",
    "两者",
    "even",
    "yours",
    "合理",
    "然则",
    "抑或",
    "于",
    "whom",
    "became",
    "跟",
    "has",
    "thorough",
    "所谓",
    "once",
    "以至",
    "那儿",
    "取得",
    "在",
    "即令",
    "哪边",
    "清楚",
    "能否",
    "currently",
    "不但",
    "can",
    "那样",
    "过来",
    "other",
    "如下",
    "原来",
    "最好",
    "reasonably",
    "becoming",
    "yes",
    "over",
    "vs",
    "那些",
    "they'll",
    "namely",
    "比较",
    "其二",
    "welcome",
    "所以",
    "虽然",
    "叫做",
    "某",
    "联系",
    "anywhere",
    "更加",
    "何况",
    "彼此",
    "despite",
    "自身",
    "自个儿",
    "们",
    "嘿",
    "哈",
    "反映",
    "此间",
    "不能",
    "据",
    "之",
    "乌乎",
    "described",
    "将",
    "后来",
    "他人",
    "重大",
    "whereupon",
    "had",
    "谁知",
    "certainly",
    "各地",
    "其中",
    "哪样",
    "sent",
    "呢",
    "followed",
    "up",
    "问题",
    "多次",
    "全部",
    "在下",
    "nothing",
    "think",
    "遇到",
    "啦",
    "any",
    "宁愿",
    "通过",
    "whatever",
    "得",
    "行为",
    "and",
    "in",
    "有关",
    "they've",
    "while",
    "形成",
    "今後",
    "积极",
    "少数",
    "严重",
    "要不是",
    "这里",
    "herein",
    "连",
    "练习",
    "whence",
    "this",
    "叮咚",
    "管",
    "yourself",
    "成为",
    "这么些",
    "并不是",
    "没有",
    "接着",
    "many",
    "que",
    "tries",
    "seems",
    "过去",
    "万一",
    "十分",
    "第",
    "若非",
    "这样",
    "以後",
    "formerly",
    "基本",
    "普通",
    "但是",
    "everywhere",
    "t's",
    "何",
    "anyways",
    "什么样",
    "特殊",
    "为",
    "nowhere",
    "能",
    "were",
    "照",
    "不断",
    "具体",
    "适用",
    "避免",
    "不独",
    "嗬",
    "怎么办",
    "uses",
    "co",
    "oh",
    "怎么",
    "certain",
    "seemed",
    "其实",
    "采取",
    "也是",
    "across",
    "ZT",
    "get",
    "固然",
    "趁着",
    "啪达",
    "反应",
    "全面",
    "同时",
    "故",
    "thanx",
    "再者",
    "有所",
    "比",
    "虽说",
    "怎",
    "实现",
    "under",
    "specify",
    "何时",
    "到",
    "相反",
    "somewhat",
    "often",
    "how",
    "com",
    "tends",
    "those",
    "除",
    "former",
    "具体地说",
    "因而",
    "由此可见",
    "seen",
    "同一",
    "wants",
    "下面",
    "常常",
    "一面",
    "plus",
    "觉得",
    "具有",
    "wherever",
    "being",
    "instead",
    "并且",
    "un",
    "者",
    "must",
    "it'll",
    "获得",
    "边",
    "elsewhere",
    "沿",
    "也",
    "insofar",
    "accordingly",
    "better",
    "根本",
    "placed",
    "对于",
    "others",
    "这个",
    "necessary",
    "于是",
    "beforehand",
    "而且",
    "似乎",
    "unless",
    "以致",
    "上面",
    "加以",
    "代替",
    "mostly",
    "ignored",
    "不管",
    "thru",
    "within",
    "遭到",
    "自从",
    "不单",
    "whereby",
    "ie",
    "or",
    "unfortunately",
    "较之",
    "indeed",
    "selves",
    "不会",
    "而外",
    "设使",
    "构成",
    "正在",
    "而",
    "依靠",
    "eg",
    "来",
    "taken",
    "唉",
    "某个",
    "too",
    "－－",
    "hasn't",
    "自家",
    "during",
    "允许",
    "would",
    "probably",
    "seeing",
    "一旦",
    "to",
    "不足",
    "往往",
    "相同",
    "当时",
    "like",
    "gone",
    "按照",
    "是否",
    "secondly",
    "latterly",
    "zt",
    "里面",
    "这儿",
    "allow",
    "啥",
    "beside",
    "哗",
    "召开",
    "之一",
    "咱",
    "doesn't",
    "开外",
    "keeps",
    "nine",
    "越是",
    "someone",
    "最后",
    "满足",
    "纵",
    "ex",
    "乃至",
    "这",
    "weren't",
    "云云",
    "移动",
    "concerning",
    "前进",
    "总之",
    "又",
    "says",
    "going",
    "what",
    "ask",
    "朝着",
    "used",
    "需要",
    "存在",
    "its",
    "own",
    "争取",
    "大多数",
    "who's",
    "依照",
    "took",
    "朝",
    "来着",
    "more",
    "啊",
    "one",
    "好的",
    "以免",
    "只有",
    "以便",
    "said",
    "举行",
    "离",
    "all",
    "鄙人",
    "意思",
    "c'mon",
    "说明",
    "凭",
    "really",
    "ZZ",
    "merely",
    "thereupon",
    "available",
    "一下",
    "加强",
    "they're",
    "它",
    "每年",
    "though",
    "虽则",
    "向",
    "连同",
    "前后",
    "和",
    "sometimes",
    "同样",
    "causes",
    "name",
    "经过",
    "viz",
    "顺着",
    "上升",
    "done",
    "per",
    "然后",
    "对",
    "整个",
    "appreciate",
    "坚决",
    "mainly",
    "个",
    "theres",
    "方便",
    "less",
    "部分",
    "假如",
    "此",
    "相当",
    "itself",
    "是的",
    "what's",
    "使得",
    "以外",
    "there",
    "your",
    "必然",
    "还有",
    "他们",
    "喂",
    "inc",
    "赶",
    "cause",
    "变成",
    "是不是",
    "let's",
    "away",
    "near",
    "不特",
    "unlikely",
    "各种",
    "与其",
    "相应",
    "liked",
    "造成",
    "considering",
    "that's",
    "normally",
    "lest",
    "looking",
    "regarding",
    "鉴于",
    "particular",
    "认为",
    "howbeit",
    "second",
    "对应",
    "通常",
    "只要",
    "这么样",
    "maybe",
    "进行",
    "zero",
    "getting",
    "这种",
    "适应",
    "nearly",
    "坚持",
    "掌握",
    "haven't",
    "five",
    "甚而",
    "人们",
    "我们",
    "出现",
    "中间",
    "可是",
    "that",
    "which",
    "为什麽",
    "倘然",
    "after",
    "could",
    "认真",
    "着",
    "诸位",
    "结合",
    "是",
    "突出",
    "hers",
    "regardless",
    "以下",
    "处理",
    "but",
    "乘",
    "它们",
    "企图",
    "周围",
    "今后",
    "宁",
    "一定",
    "乃",
    "一切",
    "故此",
    "要是",
    "分别",
    "也罢",
    "using",
    "并不",
    "一则",
    "也好",
    "之後",
    "要不然",
    "rd",
    "哪年",
    "毫不",
    "任何",
    "couldn't",
    "hereafter",
    "总的说来",
    "那么样",
    "拿",
    "一来",
    "不得",
    "大约",
    "范围",
    "you've",
    "保持",
    "帮助",
    "off",
    "?",
    "will",
    "啐",
    "keep",
    "themselves",
    "哼唷",
    "重要",
    "whenever",
    "truly",
    "use",
    "mean",
    "一方面",
    "呕",
    "别说",
    "furthermore",
    "inward",
    "isn't",
    "说说",
    "very",
    "twice",
    "出来",
    "一边",
    "again",
    "即若",
    "不惟",
    "但",
    "从而",
    "she",
    "他的",
    "sometime",
    "相等",
    "任务",
    "以前",
    "极了",
    "呃",
    "his",
    "saying",
    "它们的",
    "不成",
    "准备",
    "--",
    "therein",
    "awfully",
    "thereby",
    "two",
}
